@@ -153,10 +153,13 @@ def plot_solves():
                 assert isinstance(result, math.SolveInfo)
                 from phi.math._tensors import disassemble_tree
                 _, (residual,) = disassemble_tree(result.residual)
-                residual_mean = math.mean(math.abs(residual), residual.shape.without('trajectory'))
+                residual_mse = math.mean(math.sqrt(math.sum(residual ** 2)), residual.shape.without('trajectory'))
+                residual_mse_max = math.max(math.sqrt(math.sum(residual ** 2)), residual.shape.without('trajectory'))
+                # residual_mean = math.mean(math.abs(residual), residual.shape.without('trajectory'))
                 residual_max = math.max(math.abs(residual), residual.shape.without('trajectory'))
-                pylab.plot(residual_mean.numpy(), label=f"{i}: {result.method}", color=cycle[i % len(cycle)])
-                pylab.plot(residual_max.numpy(), alpha=0.2, color=cycle[i % len(cycle)])
+                pylab.plot(residual_mse.numpy(), label=f"{i}: {result.method}", color=cycle[i % len(cycle)])
+                pylab.plot(residual_max.numpy(), '--', alpha=0.2, color=cycle[i % len(cycle)])
+                pylab.plot(residual_mse_max.numpy(), alpha=0.2, color=cycle[i % len(cycle)])
                 print(f"Solve {i}: {result.method} ({1000 * result.solve_time:.1f} ms)\n"
                       f"\t{result.solve}\n"
                       f"\t{result.msg}\n"
@@ -165,7 +168,7 @@ def plot_solves():
                       f"\tIterations: {result.iterations}\n"
                       f"\tFunction evaulations: {result.function_evaluations.trajectory[-1]}")
             pylab.yscale('log')
-            pylab.ylabel("Mean / Max Residual")
+            pylab.ylabel("Residual: MSE / max / individual max")
             pylab.xlabel("Iteration")
             pylab.title(f"Solve Convergence")
             pylab.legend(loc='upper right')
