@@ -5,8 +5,8 @@ from phi.geom import Box, Geometry, GridCell
 from . import HardGeometryMask
 from ._field import SampledField, Field, sample, reduce_sample
 from ..geom._stack import GeometryStack
-from ..math import Shape
-from ..math._shape import spatial, channel
+from ..math import Shape, NUMPY
+from ..math._shape import spatial, channel, parse_dim_order
 from ..math._tensors import TensorStack, Tensor
 
 
@@ -16,6 +16,9 @@ class Grid(SampledField):
     """
 
     def __init__(self, elements: Geometry, values: Tensor, extrapolation: float or math.Extrapolation, resolution: Shape, bounds: Box):
+        if bounds.size.vector.item_names is None:
+            with NUMPY:
+                bounds = bounds.shifted(math.zeros(channel(vector=spatial(values).names)))
         SampledField.__init__(self, elements, values, extrapolation, bounds)
         assert values.shape.spatial_rank == elements.spatial_rank, f"Spatial dimensions of values ({values.shape}) do not match elements {elements}"
         assert values.shape.spatial_rank == bounds.spatial_rank, f"Spatial dimensions of values ({values.shape}) do not match elements {elements}"
@@ -142,6 +145,7 @@ class CenteredGrid(Grid):
                 * `tuple` or `list`: interprets the sequence as vector, used for all sample points
                 * `phi.math.Tensor` compatible with grid dims: uses tensor values as grid values
                 * Function `values(x)` where `x` is a `phi.math.Tensor` representing the physical location.
+                    The spatial dimensions of the grid will be passed as batch dimensions to the function.
 
             extrapolation: The grid extrapolation determines the value outside the `values` tensor.
                 Allowed types: `float`, `phi.math.Tensor`, `phi.math.extrapolation.Extrapolation`.
@@ -168,7 +172,7 @@ class CenteredGrid(Grid):
             elif isinstance(values, Field):
                 values = reduce_sample(values, elements)
             elif callable(values):
-                values = values(elements.center)
+                values = math.map_s2b(values)(elements.center)
                 assert isinstance(values, math.Tensor), f"values function must return a Tensor but returned {type(values)}"
             else:
                 if isinstance(values, (tuple, list)) and len(values) == resolution.rank:
@@ -232,7 +236,6 @@ class StaggeredGrid(Grid):
     
     Staggered grids support batch and spatial dimensions but only one channel dimension for the staggered vector components.
 
-
     See Also:
         `CenteredGrid`,
         `Grid`,
@@ -260,6 +263,7 @@ class StaggeredGrid(Grid):
                   Must contain a `vector` dimension with each slice consisting of one more element along the dimension they describe.
                   Use `phi.math.stack()` to manually create this non-uniform tensor.
                 * Function `values(x)` where `x` is a `phi.math.Tensor` representing the physical location.
+                    The spatial dimensions of the grid will be passed as batch dimensions to the function.
 
             extrapolation: The grid extrapolation determines the value outside the `values` tensor.
                 Allowed types: `float`, `phi.math.Tensor`, `phi.math.extrapolation.Extrapolation`.
@@ -292,7 +296,7 @@ class StaggeredGrid(Grid):
             elif isinstance(values, Field):
                 values = reduce_sample(values, elements)
             elif callable(values):
-                values = values(elements.center)
+                values = math.map_s2b(values)(elements.center)
                 if elements.shape.shape.rank > 1:  # Different number of X and Y faces
                     assert isinstance(values, TensorStack), f"values function must return a staggered Tensor but returned {type(values)}"
                 assert 'staggered_direction' in values.shape
@@ -362,12 +366,19 @@ class StaggeredGrid(Grid):
         extrapolation = self._extrapolation[item]
         bounds = GridCell(self._resolution, self._bounds)[item].bounds
         if 'vector' in item:
-            if isinstance(item['vector'], int):
-                dim = self.shape.spatial.names[item['vector']]
+            selection = item['vector']
+            if isinstance(selection, str) and ',' in selection:
+                selection = parse_dim_order(selection)
+            if isinstance(selection, str):  # single item name
+                item_names = self.shape.get_item_names('vector', fallback_spatial=True)
+                assert selection in item_names, f"Accessing field.vector['{selection}'] failed. Item names are {item_names}."
+                selection = item_names.index(selection)
+            if isinstance(selection, int):
+                dim = self.shape.spatial.names[selection]
                 comp_cells = GridCell(self.resolution, bounds).stagger(dim, *self.extrapolation.valid_outer_faces(dim))
                 return CenteredGrid(values, bounds=comp_cells.bounds, extrapolation=extrapolation)
             else:
-                assert isinstance(item['vector'], slice) and not item['vector'].start and not item['vector'].stop
+                assert isinstance(selection, slice) and not selection.start and not selection.stop
         return StaggeredGrid(values, bounds=bounds, extrapolation=extrapolation)
 
     def staggered_tensor(self) -> Tensor:
@@ -414,10 +425,11 @@ def staggered_elements(resolution: Shape, bounds: Box, extrapolation: math.Extra
     for dim in resolution.names:
         lower, upper = extrapolation.valid_outer_faces(dim)
         grids.append(cells.stagger(dim, lower, upper))
-    return geom.stack(grids, channel('staggered_direction'))
+    return geom.stack(grids, channel(staggered_direction=resolution.names))
 
 
 def expand_staggered(values: Tensor, resolution: Shape, extrapolation: math.Extrapolation):
+    """ Add missing spatial dimensions to `values` """
     cells = GridCell(resolution, Box(0, math.wrap((1,) * resolution.rank, channel(vector=resolution.names))))
     components = values.vector.unstack(resolution.spatial_rank)
     tensors = []
