@@ -166,10 +166,11 @@ class Box(BaseBox, metaclass=BoxType):
     Boxes can be constructed either from two positional vector arguments `(lower, upper)` or by specifying the limits by dimension name as `kwargs`.
 
     **Examples**:
-        ```python
-        Box(x=1, y=1)  # creates a two-dimensional unit box with `lower=(0, 0)` and `upper=(1, 1)`.
-        Box(x=(None, 1), y=(0, None)  # creates a Box with `lower=(-inf, 0)` and `upper=(1, inf)`.
-        ```
+
+    ```python
+    Box(x=1, y=1)  # creates a two-dimensional unit box with `lower=(0, 0)` and `upper=(1, 1)`.
+    Box(x=(None, 1), y=(0, None)  # creates a Box with `lower=(-inf, 0)` and `upper=(1, inf)`.
+    ```
     """
 
     def __init__(self,
@@ -205,6 +206,9 @@ class Box(BaseBox, metaclass=BoxType):
             upper = [INF if u is None else u for u in upper]
             self._upper = math.wrap(upper, math.channel(vector=tuple(size.keys())))
             self._lower = math.wrap(lower, math.channel(vector=tuple(size.keys())))
+        vector_shape = self._lower.shape & self._upper.shape
+        self._lower = math.expand(self._lower, vector_shape)
+        self._upper = math.expand(self._upper, vector_shape)
         if self.size.vector.item_names is None:
             warnings.warn("Creating a Box without item names prevents certain operations like project()", DeprecationWarning, stacklevel=2)
 
