@@ -17,6 +17,10 @@ from phi.math.backend._backend import combined_dim, SolveResult, PHI_LOGGER
 from ..math.backend._dtype import to_numpy_dtype, from_numpy_dtype
 
 
+from jax.config import config
+config.update("jax_enable_x64", True)
+
+
 class JaxBackend(Backend):
 
     def __init__(self):
@@ -166,17 +170,14 @@ class JaxBackend(Backend):
             @wraps(f)
             def aux_f(*args):
                 output = f(*args)
-                if isinstance(output, (tuple, list)) and len(output) == 1:
-                    output = output[0]
-                result = (output[0], output[1:]) if isinstance(output, (tuple, list)) else (output, None)
-                if result[0].ndim > 0:
-                    result = jnp.sum(result[0]), result[1]
-                return result
+                output = output if isinstance(output, (tuple, list)) else [output]
+                return jnp.sum(output[0]), output
             jax_grad_f = jax.value_and_grad(aux_f, argnums=wrt, has_aux=True)
             @wraps(f)
             def unwrap_outputs(*args):
-                (loss, aux), grads = jax_grad_f(*args)
-                return (loss, *aux, *grads) if aux is not None else (loss, *grads)
+                args = [self.to_float(arg) if self.dtype(arg).kind in (bool, int) else arg for arg in args]
+                (_, output_tuple), grads = jax_grad_f(*args)
+                return (*output_tuple, *grads)
             return unwrap_outputs
         else:
             @wraps(f)
@@ -227,10 +228,11 @@ class JaxBackend(Backend):
             raise ValueError(dtype)
         return jax.device_put(tensor, self._default_device.ref)
 
-    def random_normal(self, shape):
+    def random_normal(self, shape, dtype: DType):
         self._check_float64()
         self.rnd_key, subkey = jax.random.split(self.rnd_key)
-        return jax.device_put(random.normal(subkey, shape, dtype=to_numpy_dtype(self.float_type)), self._default_device.ref)
+        dtype = dtype or self.float_type
+        return jax.device_put(random.normal(subkey, shape, dtype=to_numpy_dtype(dtype)), self._default_device.ref)
 
     def range(self, start, limit=None, delta=1, dtype: DType = DType(int, 32)):
         if limit is None:
