@@ -334,12 +334,23 @@ def print_(obj: Tensor or TensorLike or Number or tuple or list or None = None,
         _print_tensor(value, name)
 
 
-def _print_tensor(value: Tensor, name: str or None):
+def _print_tensor(value: Tensor, name: str or None, color=True):
+    if color:
+        v = '\033[94m'  # value
+        s = '\033[92m'  # shape
+        e = '\033[0m'   # end
+        d = '\033[93m'  # dtype
+        g = '\033[37m'  # grey (additional)
+        # BOLD = '\033[1m'
+        # UNDERLINE = '\033[4m'
+    else:
+        v, s, d, e, g = '', '', '', '', ''
+
     if name:
-        print(" " * 16 + name)
+        print(" " * 16 + f"{d}{name}{e}")
     dim_order = tuple(sorted(value.shape.spatial.names, reverse=True))
     if value.shape.spatial_rank == 0:
-        print(f"shape={value.shape}")
+        print(f"{s}{value.shape}{e}")
         if value.shape.rank <= 1:
             text = np.array2string(value.numpy(), precision=2, separator=', ', max_line_width=np.inf)
             print(' ' + re.sub('[\\[\\]]', '', text))
@@ -347,15 +358,15 @@ def _print_tensor(value: Tensor, name: str or None):
             text = np.array2string(value.numpy(value.shape), precision=2, separator=', ', max_line_width=np.inf)
             print(text)
     elif value.shape.spatial_rank == 1:
-        for index_dict in value.shape.non_spatial.meshgrid():
+        for index_dict in value.shape.non_spatial.meshgrid(names=True):
             if value.shape.non_spatial.volume > 1:
-                print(f"--- {', '.join('%s=%d' % (name, idx) for name, idx in index_dict.items())} ---")
+                print(f"---{s} {', '.join(f'{name}={idx}' for name, idx in index_dict.items())} {e}---")
             text = np.array2string(value[index_dict].numpy(dim_order), precision=2, separator=', ', max_line_width=np.inf)
             print(' ' + re.sub('[\\[\\]]', '', text))
     elif value.shape.spatial_rank == 2:
-        for index_dict in value.shape.non_spatial.meshgrid():
+        for index_dict in value.shape.non_spatial.meshgrid(names=True):
             if value.shape.non_spatial.volume > 1:
-                print(f"--- {', '.join('%s=%d' % (name, idx) for name, idx in index_dict.items())} ---")
+                print(f"---{s} {', '.join(f'{name}={idx}' for name, idx in index_dict.items())} {e}---")
             text = np.array2string(value[index_dict].numpy(dim_order)[::-1], precision=2, separator=', ', max_line_width=np.inf)
             print(' ' + re.sub('[\\[\\]]', '', re.sub('\\],', '', text)))
     else:
@@ -471,8 +482,7 @@ def random_normal(*shape: Shape, dtype: DType = None) -> Tensor:
     """
 
     def uniform_random_normal(shape):
-        native = choose_backend(*shape.sizes, prefer_default=True).random_normal(shape.sizes)
-        native = native if dtype is None else native.astype(dtype)
+        native = choose_backend(*shape.sizes, prefer_default=True).random_normal(shape.sizes, DType.as_dtype(dtype))
         return NativeTensor(native, shape)
 
     return _initialize(uniform_random_normal, shape)
@@ -1210,6 +1220,8 @@ def std(value: Tensor or list or tuple, dim: str or int or tuple or list or None
     """
     Computes the standard deviation over `values` along the specified dimensions.
 
+    *Warning*: The standard deviation of non-uniform tensors along the stack dimension is undefined.
+
     Args:
         value: `Tensor` or `list` / `tuple` of Tensors.
         dim: Dimension or dimensions to be reduced. One of
@@ -1224,7 +1236,7 @@ def std(value: Tensor or list or tuple, dim: str or int or tuple or list or None
     Returns:
         `Tensor` without the reduced dimensions.
     """
-    return _reduce(value, dim,
+    return _reduce(cached(value), dim,
                    native_function=lambda backend, native, dim: backend.std(native, dim),
                    collapsed_function=lambda inner, red_shape: inner,
                    unaffected_function=lambda value: value * 0)
@@ -1811,6 +1823,8 @@ def boolean_mask(x: Tensor, dim: str, mask: Tensor):
     Returns:
         Selected values of `x` as `Tensor` with dimensions from `x` and `mask`.
     """
+    assert dim in mask.shape, f"mask dimension '{dim}' must be present on the mask but got {mask.shape}"
+    
     def uniform_boolean_mask(x: Tensor, mask_1d: Tensor):
         if dim in x.shape:
             x_native = x.native(x.shape.names)  # order does not matter
@@ -1851,12 +1865,12 @@ def gather(values: Tensor, indices: Tensor, dims: str or Shape or tuple or list
         indices = to_int32(indices)
     dims = parse_dim_order(dims)
     batch = (values.shape.batch & indices.shape.batch).without(dims)
-    channel = values.shape.channel.without(dims)
+    channel = values.shape.without(dims).without(batch)
     native_values = reshaped_native(values, [batch, *dims, channel])
     native_indices = reshaped_native(indices, [batch, *indices.shape.non_batch.non_channel, indices.shape.channel])
     backend = choose_backend(native_values, native_indices)
     native_result = backend.batched_gather_nd(native_values, native_indices)
-    result = reshaped_tensor(native_result, [batch, *indices.shape.non_channel.non_batch, values.shape.channel])
+    result = reshaped_tensor(native_result, [batch, *indices.shape.non_channel.non_batch, channel])
     return result
 
 
@@ -2027,7 +2041,7 @@ def dtype(x) -> DType:
         return choose_backend(x).dtype(x)
 
 
-def expand(value: Tensor, dims: Shape):
+def expand(value: float or Tensor, dims: Shape):
     """
     Adds dimensions to a `Tensor` by implicitly repeating the tensor values along the new dimensions.
     If `value` already contains some of the new dimensions, a size and type check is performed instead.
@@ -2051,8 +2065,7 @@ def expand(value: Tensor, dims: Shape):
     shape = value.shape
     for dim in reversed(dims):
         if dim in value.shape:
-            assert dim.size is None or shape.get_size(dim.name) == dim.size, f"Cannot expand tensor with shape {shape} by dimension {dim}"
-            assert shape.get_type(dim) == dim.type, f"Cannot expand tensor with shape {shape} by dimension {dim} of type '{dim.type}' because the dimension types do not match. Original type of '{dim.name}' was {shape.get_type(dim.name)}."
+            shape &= dim  # checks sizes, copies item names
         else:
             if dim.size is None:
                 dim = dim.with_sizes([1])
