@@ -139,9 +139,9 @@ class TFBackend(Backend):
                 imag = tf.cast(tf.random.uniform(shape, low.imag, high.imag, dtype=to_numpy_dtype(DType(float, dtype.precision))), tdt)
                 return real + 1j * imag
 
-    def random_normal(self, shape):
+    def random_normal(self, shape, dtype: DType):
         with self._default_device.ref:
-            return tf.random.normal(shape, dtype=to_numpy_dtype(self.float_type))
+            return tf.random.normal(shape, dtype=to_numpy_dtype(dtype or self.float_type))
 
     def range(self, start, limit=None, delta=1, dtype: DType = DType(int, 32)):
         with self._default_device.ref:
@@ -212,6 +212,8 @@ class TFBackend(Backend):
 
     def mean(self, value, axis=None, keepdims=False):
         with tf.device(value.device):
+            if self.dtype(value).kind not in (float, complex):
+                value = self.to_float(value)
             if axis is not None:
                 if not isinstance(axis, int):
                     axis = list(axis)
@@ -400,6 +402,8 @@ class TFBackend(Backend):
 
     def std(self, x, axis=None, keepdims=False):
         with tf.device(x.device):
+            if self.dtype(x).kind not in (float, complex):
+                x = self.to_float(x)
             _mean, var = tf.nn.moments(x, axis, keepdims=keepdims)
             return tf.sqrt(var)
 
@@ -413,10 +417,14 @@ class TFBackend(Backend):
 
     def any(self, boolean_tensor, axis=None, keepdims=False):
         with tf.device(boolean_tensor.device):
+            if self.dtype(boolean_tensor).kind != bool:
+                boolean_tensor = tf.not_equal(boolean_tensor, 0)
             return tf.reduce_any(boolean_tensor, axis=axis, keepdims=keepdims)
 
     def all(self, boolean_tensor, axis=None, keepdims=False):
         with tf.device(boolean_tensor.device):
+            if self.dtype(boolean_tensor).kind != bool:
+                boolean_tensor = tf.not_equal(boolean_tensor, 0)
             return tf.reduce_all(boolean_tensor, axis=axis, keepdims=keepdims)
 
     def quantile(self, x, quantiles):
@@ -623,22 +631,19 @@ class TFBackend(Backend):
         @wraps(f)
         def eval_grad(*args):
             args = [self.as_tensor(arg, True) if i in wrt else arg for i, arg in enumerate(args)]
+            args = [self.to_float(arg) if self.dtype(arg).kind in (bool, int) else arg for arg in args]
             wrt_args = [arg for i, arg in enumerate(args) if i in wrt]
             with tf.GradientTape(watch_accessed_variables=False) as tape:
                 for arg in wrt_args:
                     assert arg.dtype in (tf.float16, tf.float32, tf.float64, tf.complex64, tf.complex128), f"Gradients can only be computed for float or complex tensors but got {arg.dtype} for argument with shape {arg.shape}"
                     tape.watch(arg)
                 output = f(*args)
-            loss, aux = (output[0], output[1:]) if isinstance(output, (tuple, list)) else (output, None)
-            # if self.ndims(loss) > 0:
-            #     loss = tf.reduce_sum(loss)  # this is not needed and will cause gradients to be None
+            output = output if isinstance(output, (tuple, list)) else [output]
+            loss = output[0]  # tf.reduce_sum(output[0]) not needed and will cause gradients to be None
             grads = list(self.as_registered.call(tape.gradient, loss, wrt_args, name=f"Backpropagation"))
             assert None not in grads, f"Gradient could not be computed for wrt argument {grads.index(None)} (argument {wrt[grads.index(None)]}) with shape {wrt_args[grads.index(None)].shape}. TensorFlow returned gradient=None."
             if get_output:
-                if aux is not None:
-                    return (loss, *aux, *grads)
-                else:
-                    return (loss, *grads)
+                return (*output, *grads)
             else:
                 return grads
         return eval_grad
